@@ -1,5 +1,5 @@
 /*
- *   $Id: device-bsd44.c,v 1.11 2002/07/02 06:49:20 psavola Exp $
+ *   $Id: device-bsd44.c,v 1.13 2004/02/05 18:44:00 lutchann Exp $
  *
  *   Authors:
  *    Craig Metz		<cmetz@inner.net>
@@ -33,17 +33,18 @@ setup_deviceinfo(int sock, struct Interface *iface)
 	int nlen;
 	uint8_t *p, *end;
 	struct AdvPrefix *prefix;
+	char zero[HWADDR_MAX];
 
 	/* just allocate 8192 bytes, should be more than enough.. */
 	if (!(ifconf.ifc_buf = malloc(ifconf.ifc_len = (32 << 8))))
 	{
-		log(LOG_CRIT, "malloc failed: %s", strerror(errno));
+		flog(LOG_CRIT, "malloc failed: %s", strerror(errno));
 		goto ret;
 	}
 
 	if (ioctl(sock, SIOCGIFCONF, &ifconf) < 0)
 	{
-		log(LOG_ERR, "ioctl(SIOCGIFCONF) failed: %s(%d)", strerror(errno), errno);
+		flog(LOG_ERR, "ioctl(SIOCGIFCONF) failed: %s(%d)", strerror(errno), errno);
 		goto ret;
 	}
 
@@ -68,7 +69,7 @@ setup_deviceinfo(int sock, struct Interface *iface)
 		
 			if (((struct sockaddr_dl *)p)->sdl_alen > HWADDR_MAX)
 			{
-				log(LOG_ERR, "address length %d too big for",
+				flog(LOG_ERR, "address length %d too big for",
 					((struct sockaddr_dl *)p)->sdl_alen,
 					iface->Name);
 				goto ret;
@@ -99,13 +100,20 @@ setup_deviceinfo(int sock, struct Interface *iface)
 			dlog(LOG_DEBUG, 3, "prefix length for %s is %d", iface->Name,
 				iface->if_prefix_len);
 
+			if (iface->if_prefix_len != -1) {
+				memset(zero, 0, ((struct sockaddr_dl *)p)->sdl_alen);
+				if (!memcmp(iface->if_hwaddr, zero, ((struct sockaddr_dl *)p)->sdl_alen))
+					flog(LOG_WARNING, "WARNING, MAC address on %s is all zero!",
+						iface->Name);
+			}
+			
 			prefix = iface->AdvPrefixList;
 			while (prefix)
 			{
 				if ((iface->if_prefix_len != -1) &&
 					(iface->if_prefix_len != prefix->PrefixLen))
 				{
-					log(LOG_WARNING, "prefix length should be %d for %s",
+					flog(LOG_WARNING, "prefix length should be %d for %s",
 						iface->if_prefix_len, iface->Name);
  				}
  			
@@ -137,13 +145,13 @@ int setup_linklocal_addr(int sock, struct Interface *iface)
 	/* just allocate 8192 bytes, should be more than enough.. */
 	if (!(ifconf.ifc_buf = malloc(ifconf.ifc_len = (32 << 8))))
 	{
-		log(LOG_CRIT, "malloc failed: %s", strerror(errno));
+		flog(LOG_CRIT, "malloc failed: %s", strerror(errno));
 		goto ret;
 	}
 
 	if (ioctl(sock, SIOCGIFCONF, &ifconf) < 0)
 	{
-		log(LOG_ERR, "ioctl(SIOCGIFCONF) failed: %s(%d)", strerror(errno), errno);
+		flog(LOG_ERR, "ioctl(SIOCGIFCONF) failed: %s(%d)", strerror(errno), errno);
 		goto ret;
 	}
 
@@ -183,7 +191,7 @@ int setup_linklocal_addr(int sock, struct Interface *iface)
 	}
 
 ret:
-	log(LOG_ERR, "no linklocal address configured for %s", iface->Name);
+	flog(LOG_ERR, "no linklocal address configured for %s", iface->Name);
 	free(ifconf.ifc_buf);
 	return -1;
 }
@@ -211,7 +219,7 @@ get_v4addr(const char *ifn, unsigned int *dst)
 
 	if( ( fd = socket(AF_INET,SOCK_DGRAM,0) ) < 0 )
 	{
-		log(LOG_ERR, "create socket for IPv4 ioctl failed for %s: %s",
+		flog(LOG_ERR, "create socket for IPv4 ioctl failed for %s: %s",
 			ifn, strerror(errno));
 		return (-1);
 	}
@@ -223,7 +231,7 @@ get_v4addr(const char *ifn, unsigned int *dst)
 	
 	if (ioctl(fd, SIOCGIFADDR, &ifr) < 0)
 	{
-		log(LOG_ERR, "ioctl(SIOCGIFADDR) failed for %s: %s",
+		flog(LOG_ERR, "ioctl(SIOCGIFADDR) failed for %s: %s",
 			ifn, strerror(errno));
 		close( fd );
 		return (-1);
