@@ -1,30 +1,21 @@
 /*
- *	$Id: log.c,v 1.4 1997/01/05 19:17:41 lf Exp $
+ *	$Id: log.c,v 1.4 2004/02/05 18:44:00 lutchann Exp $
  *
  *	Authors:
  *	 Lars Fenneberg		<lf@elemental.net>	 
  *
- *	This software is Copyright 1996 by the above mentioned author(s), 
+ *	This software is Copyright 1996,1997 by the above mentioned author(s), 
  *	All Rights Reserved.
  *
  *	The license which is distributed with this software in the file
  *	COPYRIGHT applies to this software. If your distribution is missing 
- *	this file, you may request it from <lf@elemental.net>.
+ *	this file, you may request it from <lutchann@litech.org>.
  *
  */
 
-#include "config.h"
-
-#include <stdio.h>
-#include <stdarg.h>
-#include <stdlib.h>
-#include <time.h>
-#include <syslog.h>
-#include <errno.h>	/* on the Alpha/glibc stdlib.h doesn't include errno.h */
-
-#define TIME_FORMAT "%b %d %H:%M:%S"
-
-#include "radvd.h"
+#include <config.h>
+#include <includes.h>
+#include <radvd.h>
 
 static int	log_method = L_NONE;
 static char *log_ident;
@@ -54,17 +45,18 @@ log_open(int method, char *ident, char *log, int facility)
 		case L_LOGFILE:
 			if (!log)
 			{
-				fprintf(stderr, "%s: no logfile specified\n", ident);
+				fprintf(stderr, "%s: no logfile specified\n", log_ident);
 				return (-1);				
 			}
-			if ((log_file_fd = fopen(log, "a")) == NULL)
+			log_file = log;
+			if ((log_file_fd = fopen(log_file, "a")) == NULL)
 			{
-				fprintf(stderr, "%s: can't open %s: %s\n", ident, log, strerror(errno));
+				fprintf(stderr, "%s: can't open %s: %s\n", log_ident, log_file, strerror(errno));
 				return (-1);				
 			}
 			break;
 		default:
-			fprintf(stderr, "%s: unknown logging method: %d\n", ident, log_method);
+			fprintf(stderr, "%s: unknown logging method: %d\n", log_ident, log_method);
 			log_method = L_NONE;
 			return (-1);				
 	}
@@ -88,7 +80,7 @@ vlog(int prio, char *format, va_list ap)
 		case L_STDERR:
 			current = time(NULL);
 			tm = localtime(&current);
-			(void) strftime(tstamp, sizeof(tstamp), TIME_FORMAT, tm);
+			(void) strftime(tstamp, sizeof(tstamp), LOG_TIME_FORMAT, tm);
 
 			fprintf(stderr, "[%s] %s: ", tstamp, log_ident);
 	    		vfprintf(stderr, format, ap);
@@ -98,7 +90,7 @@ vlog(int prio, char *format, va_list ap)
 		case L_LOGFILE:
 			current = time(NULL);
 			tm = localtime(&current);
-			(void) strftime(tstamp, sizeof(tstamp), TIME_FORMAT, tm);
+			(void) strftime(tstamp, sizeof(tstamp), LOG_TIME_FORMAT, tm);
 
 			fprintf(log_file_fd, "[%s] %s: ", tstamp, log_ident);
     			vfprintf(log_file_fd, format, ap);
@@ -130,7 +122,7 @@ dlog(int prio, int level, char *format, ...)
 }
 
 int
-log(int prio, char *format, ...)
+flog(int prio, char *format, ...)
 {
 	va_list ap;
 	int res;
