@@ -1,132 +1,123 @@
 /*
- *   $Id: timer.c,v 1.4 1997/01/05 19:17:41 lf Exp $
+ *   $Id: timer.c,v 1.6 2001/11/14 19:58:11 lutchann Exp $
  *
  *   Authors:
  *    Pedro Roque		<roque@di.fc.ul.pt>
  *    Lars Fenneberg		<lf@elemental.net>
  *
- *   This software is Copyright 1996 by the above mentioned author(s), 
+ *   This software is Copyright 1996-2000 by the above mentioned author(s), 
  *   All Rights Reserved.
  *
  *   The license which is distributed with this software in the file COPYRIGHT
  *   applies to this software. If your distribution is missing this file, you
- *   may request it from <lf@elemental.net>.
+ *   may request it from <lutchann@litech.org>.
  *
  */
 
-#include "config.h"
-
-#include <stdio.h>
-#include <signal.h>
-#include <unistd.h>
-#include <syslog.h>
-#include <string.h>
-#include <sys/time.h>
-
-#include "radvd.h"
+#include <config.h>
+#include <includes.h>
+#include <radvd.h>
 
 static struct timer_lst timers_head = {
-	~0,
+	{LONG_MAX, LONG_MAX},
 	NULL, NULL,
 	&timers_head, &timers_head
 };
 
-static unsigned long sched_timer = 0;
-
 static void alarm_handler(int sig);
 
+static void
+schedule_timer(void)
+{
+	struct timer_lst *tm = timers_head.next;
+	struct timeval tv;
+
+	gettimeofday(&tv, NULL);
+
+	if (tm != &timers_head)
+	{
+		struct itimerval next;
+	       
+	        memset(&next, 0, sizeof(next));
+	       
+	        timersub(&tm->expires, &tv, &next.it_value);
+
+		signal(SIGALRM, alarm_handler);
+
+		if ((next.it_value.tv_sec > 0) || 
+				((next.it_value.tv_sec == 0) && (next.it_value.tv_usec > 0)))
+		{
+			dlog(LOG_DEBUG, 4, "calling alarm: %ld secs, %ld usecs", 
+					next.it_value.tv_sec, next.it_value.tv_usec);
+			setitimer(ITIMER_REAL, &next,  NULL);
+		}
+		else
+		{
+			dlog(LOG_DEBUG, 4, "next timer has already expired, queueing signal");	
+			kill(getpid(), SIGALRM);
+		}
+	}
+}
+
 void
-set_timer(struct timer_lst *tm, int secs)
+set_timer(struct timer_lst *tm, double secs)
 {
 	struct timeval tv;
 	struct timer_lst *lst;
 	sigset_t bmask, oldmask;
+	struct timeval firein;
 
-	dlog(LOG_DEBUG, 3, "setting timer: %d secs", secs);
+	dlog(LOG_DEBUG, 3, "setting timer: %.2f secs", secs);
 
-	/*
-	 * disable delivery of alarm signals
-	 */
-	sigemptyset(&bmask);
-	sigaddset(&bmask, SIGALRM);
+	firein.tv_sec = (long)secs;
+	firein.tv_usec = (long)((secs - (double)firein.tv_sec) * 1000000);
 
-	sigprocmask(SIG_BLOCK, &bmask, &oldmask);
+	dlog(LOG_DEBUG, 5, "setting timer: %ld secs %ld usecs", firein.tv_sec, firein.tv_usec);
 
 	gettimeofday(&tv, NULL);
-	tm->expires = tv.tv_sec + secs;
-	
+	timeradd(&tv, &firein, &tm->expires);
+
+	sigemptyset(&bmask);
+	sigaddset(&bmask, SIGALRM);
+	sigprocmask(SIG_BLOCK, &bmask, &oldmask);
+
 	lst = &timers_head;
 
 	do {
 		lst = lst->next;
-	} while (tm->expires > lst->expires);
+	} while ((tm->expires.tv_sec > lst->expires.tv_sec) ||
+		 ((tm->expires.tv_sec == lst->expires.tv_sec) && 
+		  (tm->expires.tv_usec > lst->expires.tv_usec)));
 
 	tm->next = lst;
 	tm->prev = lst->prev;
 	lst->prev = tm;
 	tm->prev->next = tm;
 
-	if (sched_timer == 0 || sched_timer > tm->expires)
-	{		
-		signal(SIGALRM, alarm_handler);
-		sched_timer = tm->expires;		
-		dlog(LOG_DEBUG, 4, "calling alarm: %d secs", secs);
-		alarm(secs);
-	}
+	dlog(LOG_DEBUG, 5, "calling schedule_timer from set_timer context");
+	schedule_timer();
 
-	/*
-	 * reenable alarm signals
-	 */
 	sigprocmask(SIG_SETMASK, &oldmask, NULL);
 }
 
-unsigned long
+void
 clear_timer(struct timer_lst *tm)
 {
 	sigset_t bmask, oldmask;
-	unsigned long scheduled;
 
-	/*
-	 * disable delivery of alarm signals
-	 */
 	sigemptyset(&bmask);
 	sigaddset(&bmask, SIGALRM);
-
 	sigprocmask(SIG_BLOCK, &bmask, &oldmask);
 	
-	scheduled = tm->expires;
-		
 	tm->prev->next = tm->next;
 	tm->next->prev = tm->prev;
 	
 	tm->prev = tm->next = NULL;
 	
-	tm = timers_head.next;
+	dlog(LOG_DEBUG, 5, "calling schedule_timer from clear_timer context");
+	schedule_timer();
 
-	if (tm != &timers_head)
-	{
-		struct timeval tv;
-		long secs;
-	
-		gettimeofday(&tv, NULL);
-		secs = tv.tv_sec - tm->expires;
-		sched_timer = tm->expires;
-
-		if (secs < 0)
-			secs = 0;
-
-		dlog(LOG_DEBUG, 4, "calling alarm: %d secs", secs);
-		alarm(secs);
-	}
-	else
-		sched_timer = 0;
-
-	/*
-	 * reenable alarm signals
-	 */
 	sigprocmask(SIG_SETMASK, &oldmask, NULL);
-
-	return scheduled;
 }
 
 static void
@@ -135,11 +126,12 @@ alarm_handler(int sig)
 	struct timer_lst *tm, *back;
 	struct timeval tv;
 
+	gettimeofday(&tv, NULL);
 	tm = timers_head.next;
 
-	gettimeofday(&tv, NULL);
-
-	while (tm->expires <= tv.tv_sec)
+	while ((tm->expires.tv_sec < tv.tv_sec)
+			|| ((tm->expires.tv_sec == tv.tv_sec) 
+			    && (tm->expires.tv_usec <= tv.tv_usec)))
 	{		
 		tm->prev->next = tm->next;
 		tm->next->prev = tm->prev;
@@ -151,28 +143,11 @@ alarm_handler(int sig)
 		(*back->handler)(back->data);
 	}
 
-	tm = timers_head.next;
-
-	if (tm != &timers_head)
-	{
-		long secs;
-	       
-		secs = tm->expires - tv.tv_sec;
-		sched_timer = tm->expires;
-
-		if (secs <= 0)
-		{
-			secs = 1;
-		}
-
-		signal(SIGALRM, alarm_handler);
-		dlog(LOG_DEBUG, 4, "calling alarm: %d secs", secs);
-		alarm(secs);
-	}
-	else
-		sched_timer = 0;
+	dlog(LOG_DEBUG, 5, "calling schedule_timer from alarm_handler context");
+	schedule_timer();
 }
 
+
 void
 init_timer(struct timer_lst *tm, void (*handler)(void *), void *data)
 {
