@@ -1,46 +1,37 @@
 /*
- *   $Id: util.c,v 1.5 1997/01/16 21:14:38 lf Exp $
+ *   $Id: util.c,v 1.4 2005/02/15 07:44:06 psavola Exp $
  *
  *   Authors:
  *    Lars Fenneberg		<lf@elemental.net>	 
  *
- *   This software is Copyright 1996 by the above mentioned author(s), 
+ *   This software is Copyright 1996,1997 by the above mentioned author(s), 
  *   All Rights Reserved.
  *
  *   The license which is distributed with this software in the file COPYRIGHT
  *   applies to this software. If your distribution is missing this file, you
- *   may request it from <lf@elemental.net>.
+ *   may request it from <lutchann@litech.org>.
  *
  */
 
-#include "config.h"
-
-#include <sys/time.h>
-#include <sys/types.h>
-#include <stdlib.h>
-#include <unistd.h>
-#include <netdb.h>
-
-#include <sys/socket.h>
-#include <netinet6/in6.h>
-
-#include "radvd.h"
+#include <config.h>
+#include <includes.h>
+#include <radvd.h>
                
 void
 mdelay(int msecs)
 {
 	struct timeval tv;
                 
-	tv.tv_sec = (int) msecs / 1000;
-	tv.tv_usec = (msecs % 1000) * 1000;
-                                
+	tv.tv_sec = (time_t)(msecs / 1000.0);
+	tv.tv_usec = (suseconds_t)((msecs - tv.tv_sec * 1000.0) * 1000.0);
+
 	select(0,(fd_set *)NULL,(fd_set *)NULL,(fd_set *)NULL, &tv);
 }
 
-int
-rand_between(int lower, int upper)
+double
+rand_between(double lower, double upper)
 {
-	return  (lower+(int)((double)(upper-lower+1)*rand()/(RAND_MAX+1.0)));
+	return ((upper - lower) / (RAND_MAX + 1.0) * rand() + lower);
 }
 
 void
@@ -48,6 +39,7 @@ print_addr(struct in6_addr *addr, char *str)
 {
 	const char *res;
 
+	/* XXX: overflows 'str' if it isn't big enough */
 	res = inet_ntop(AF_INET6, (void *)addr, str, INET6_ADDRSTRLEN);
 	
 	if (res == NULL) 
